// this will kill users who fall below a
// minimum speed..
// compile line: cc -o minspeed minspeed.c
// and copy it to /glftpd/bin
// to run from crontab:
// put this line in your crontab
// 0-59/5 * * * * /glftpd/bin/minspeed
// to run from glFtpD:
// add site_cmd EXEC /bin/minspeed
// add custom-minspeed 1
// to glftpd.conf.. now users with flag 1 can
// kill people who are too slow
// problems: romracer@isos.net

#define KEY	0x0000DEAD   // Default KEY used by DAEMON

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <syslog.h>
#include <unistd.h>
#include <sys/param.h>

struct ONLINE
{
  char   tagline[64];
  char   username[24];
  char   status[256];
  char   host[256];
  char   currentdir[256];
  long   groupid;
  time_t login_time;
  struct timeval tstart;   
  unsigned long bytes_xfer; 
  pid_t  procid;
};

// BEGIN MODIFY
// setting to zero disables that check
// mindlspeed = min speed allowed to dl
// minulspeed = min speed allowed to ul
// watch minulspeed if youre on cable

const mindlspeed = 50;
const minulspeed = 50;

// END MODIFY


static struct ONLINE *online;
static int  num_users = 0;
static int  shmid;
static struct shmid_ds ipcbuf;

static void quit();
static double calc_time();

static void
checkusers(void)
{
  register int i = 0;
  char statbuf[ 500 ];
  char pidbuf[ 10 ];
  char filename[20];
			
  for (i=0; i<num_users; i++) 
    {
      if (online[i].procid == 0)
        continue;
		
      if (!strncasecmp (online[i].status, "STOR", 4) || !strncasecmp (online[i].status, "APPE", 4) && online[i].bytes_xfer != 0) {
          if ( calc_time(i) <= minulspeed ) {
             printf("%s is uploading too slow\n", online[i].username);
             kill(online[i].procid, 9);
          }
        }
      else if (!strncasecmp (online[i].status, "RETR", 4) && online[i].bytes_xfer != 0)
        {
          if ( calc_time(i) <= mindlspeed ) {
             printf("%s is downloading too slow\n", online[i].username);
             kill(online[i].procid, 9);
          }
        }
    }
  return;
}

int
main( int argc, char **argv )
{

  if ((shmid = shmget( (key_t)KEY, 0, 0))
       == -1)
    {
      fprintf(stdout, "No Users Currently On Site!\n");
      exit(1);
    }

  if ((online = (struct ONLINE *)shmat( shmid, NULL, SHM_RDONLY)) == (struct ONLINE *)-1)
    {
      fprintf(stdout, "Error: (SHMAT) Failed!\n");
      exit(1);
    }
							     
  shmctl( shmid, IPC_STAT, &ipcbuf);
  num_users = ipcbuf.shm_segsz / sizeof( struct ONLINE );

  checkusers();
   
  quit( 0 );
}

static double
calc_time(int pid)
{
  struct timeval tstop;
  double delta, rate;

  if (online[pid].bytes_xfer < 1)
    return 0;
  gettimeofday(&tstop, (struct timezone *)0 );
  delta = ((tstop.tv_sec*10.)+(tstop.tv_usec/100000.)) -
         ((online[pid].tstart.tv_sec*10.)+(online[pid].tstart.tv_usec/100000.));
  delta = delta/10.;
  rate = ((online[pid].bytes_xfer / 1024.0) / (delta));
  if (!rate) rate++;
  return (double)(rate);
}

static void
quit(int exit_status)
{
  shmctl( shmid, IPC_STAT, &ipcbuf);
  if (ipcbuf.shm_nattch <= 1) {
    shmctl( shmid, IPC_RMID, 0);
  }
  shmdt(0);
  exit(exit_status);
}
